package com.zjyr.beidouservice.common;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;

/**
 * The type Common enum handler.
 *
 * @param <E> the type parameter
 */
public final class CommonEnumHandler<E extends EnumerationBase> extends BaseTypeHandler<E> {
    /**
     * The Enum type.
     */
    private final Class<E> enumType;
    /**
     * The Enums.
     */
    private final List<E>  enums;

    /**
     * Instantiates a new Common enum handler.
     *
     * @param type the type
     */
    public CommonEnumHandler(Class<E> type) {
        if (type == null) {
            throw new IllegalArgumentException("Type argument cannot be null");
        }

        this.enumType = type;
        this.enums    = Arrays.asList(type.getEnumConstants());
    }

    /**
     * Code of e.
     *
     * @param <E>       the type parameter
     * @param enumClass the enum class
     * @param code      the code
     * @return the e
     */
    public static <E extends Enum<?> & EnumerationBase> E codeOf(Class<E> enumClass, int code) {
        E[] enumCodes = enumClass.getEnumConstants();

        for (E e : enumCodes) {
            if (e.getValue() == code) {
                return e;
            }
        }

        return null;
    }

    @Override
    public void setNonNullParameter(PreparedStatement preparedStatement,
                                    int i,
                                    E e,
                                    JdbcType jdbcType) throws SQLException {
        preparedStatement.setInt(i, e.getValue());
    }

    @Override
    public E getNullableResult(ResultSet resultSet, String s) throws SQLException {
        if (resultSet.getObject(s) == null) {
            return null;
        }

        int val = resultSet.getInt(s);
        return locateEnumStatus(val);
    }

    @Override
    public E getNullableResult(ResultSet resultSet, int index) throws SQLException {
        if (resultSet.getObject(index) == null) {
            return null;
        }

        int val = resultSet.getInt(index);
        return locateEnumStatus(val);
    }

    @Override
    public E getNullableResult(CallableStatement callableStatement, int index) throws SQLException {
        if (callableStatement.getObject(index) == null) {
            return null;
        }

        int val = callableStatement.getInt(index);
        return locateEnumStatus(val);
    }

    /**
     * Locate enum status e.
     *
     * @param index the index
     * @return the e
     */
    private E locateEnumStatus(int index) {
        for (E e : enums) {
            if (e.getValue() == index) {
                return e;
            }
        }
        throw new IllegalArgumentException(enumType.getName() + "  unknown enumerated type  index:" + index);
    }
}
